/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#ifndef GRAPHIC_GRAPHICMASTER_H
#define GRAPHIC_GRAPHICMASTER_H

#include "polygraphic.h"

#include <OS/list.h>

declarePtrList(GraphicList,Graphic)

class GlyphViewer;

class GraphicMaster: public PolyGraphic
{
	public:
		GraphicMaster(GlyphViewer *,Graphic * = nil,const Color * = nil);
		virtual ~GraphicMaster();

		void background(const Color *);
		const Color* background() const;

		virtual void request(Requisition &) const;
		virtual void allocate(Canvas *,const Allocation &,Extension &);

		virtual Glyph *clone() const;

		GraphicList *selection() const;
		void get_sbounds(Coord &,Coord &,Coord &,Coord &) const;
		boolean is_selected(const Graphic *) const;
		boolean select(Graphic *,Canvas * =nil,boolean =false);
		void unselect(Graphic *);
		void clear_selection();
		void transform_selection(const Transformer *);

		GlyphViewer *glyphviewer() const; // inline
		void glyphviewer(GlyphViewer *);

		void centerpoint(Coord &x,Coord &y) const;

		virtual void update(GlyphViewer * = nil);

		virtual ClassId get_classid() const;
		virtual boolean is_a(ClassId) const;
	protected:
		virtual void draw_gs(Canvas *,Graphic *);
		virtual void drawclipped_gs(Canvas *,Coord,Coord,Coord,Coord,Graphic *);
	protected:
		GlyphViewer *_gv;
		GraphicList *_gr_list;
		const Color *_bg;
		Allocation _a;
	private:
		void draw_frame(Canvas *);
};

inline const Color *GraphicMaster::background() const
{
	return _bg;
}

inline GraphicList *GraphicMaster::selection() const
{
	return _gr_list;
}

inline GlyphViewer *GraphicMaster::glyphviewer() const
{
	return _gv;
}

#endif
