/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/ellipses.h>

#include <InterViews/brush.h>
#include <InterViews/color.h>
#include <InterViews/transformer.h>

static float p0=1.00000000;
static float p1=0.89657547;  // cos 30  *sqrt(1+tan 15  *tan 15)
static float p2=0.70710678;  // cos 45 
static float p3=0.51763809;  // cos 60  *sqrt(1+tan 15  *tan 15)
static float p4=0.26794919;  // tan 15

Circle::Circle(const Brush *brush,const Color *stroke,const Color *fill,Coord x,Coord y,Coord r,Transformer *t):
	Graphic(brush,stroke,fill,nil,true,true,25,t)
{
	float px0=p0*r,py0=p0*r;
	float px1=p1*r,py1=p1*r;
	float px2=p2*r,py2=p2*r;
	float px3=p3*r,py3=p3*r;
	float px4=p4*r,py4=p4*r;
	
	add_point(x+r,y);
	add_curve(x+px2,y+py2,x+px0,y+py4,x+px1,y+py3);
	add_curve(x,y+r,x+px3,y+py1,x+px4,y+py0);
	add_curve(x-px2,y+py2,x-px4,y+py0,x-px3,y+py1);
	add_curve(x-r,y,x-px1,y+py3,x-px0,y+py4);
	add_curve(x-px2,y-py2,x-px0,y-py4,x-px1,y-py3);
	add_curve(x,y-r,x-px3,y-py1,x-px4,y-py0);
	add_curve(x+px2,y-py2,x+px4,y-py0,x+px3,y-py1);
	add_curve(x+r,y,x+px1,y-py3,x+px0,y-py4);
}

Circle::~Circle()
{
}

Glyph *Circle::clone() const
{
	Coord r=_x[0]-_x[6];
	Coord x=_x[6];
	Coord y=_y[0];
	return new Circle(_brush,_stroke,_fill,x,y,r,_t);
}

ClassId Circle::get_classid() const
{
	return CIRCLE_GRAPHIC;
}

boolean Circle::is_a(ClassId id) const
{
	return id==CIRCLE_GRAPHIC || Graphic::is_a(id);
}

//
//
//

Ellipse::Ellipse(const Brush *brush,const Color *stroke,const Color *fill,Coord x,Coord y,Coord rx,Coord ry,Transformer *t):
	Graphic(brush,stroke,fill,nil,true,true,25,t)
{
	float px0=p0*rx,py0=p0*ry;
	float px1=p1*rx,py1=p1*ry;
	float px2=p2*rx,py2=p2*ry;
	float px3=p3*rx,py3=p3*ry;
	float px4=p4*rx,py4=p4*ry;
	
	add_point(x+rx,y);
	add_curve(x+px2,y+py2,x+px0,y+py4,x+px1,y+py3);
	add_curve(x,y+ry,x+px3,y+py1,x+px4,y+py0);
	add_curve(x-px2,y+py2,x-px4,y+py0,x-px3,y+py1);
	add_curve(x-rx,y,x-px1,y+py3,x-px0,y+py4);
	add_curve(x-px2,y-py2,x-px0,y-py4,x-px1,y-py3);
	add_curve(x,y-ry,x-px3,y-py1,x-px4,y-py0);
	add_curve(x+px2,y-py2,x+px4,y-py0,x+px3,y-py1);
	add_curve(x+rx,y,x+px1,y-py3,x+px0,y-py4);
}

Ellipse::~Ellipse()
{
}

Glyph *Ellipse::clone() const
{
	Coord rx=_x[0]-_x[4];
	Coord ry=_y[4]-_x[0];
	Coord x=_x[4];
	Coord y=_y[0];
	return new Ellipse(_brush,_stroke,_fill,x,y,rx,ry,_t);
}

ClassId Ellipse::get_classid() const
{
	return ELLIPSE_GRAPHIC;
}

boolean Ellipse::is_a(ClassId id) const
{
	return id==ELLIPSE_GRAPHIC || Graphic::is_a(id);
}

