/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/graphicmaster.h>
#include <Graphic/glyphviewer.h>
#include <Graphic/handles.h>
#include <Graphic/slrect.h>
#include <Graphic/menutool.h>
#include <Graphic/globals.h>

#include <IV-look/kit.h>

#include <InterViews/canvas.h>
#include <InterViews/display.h>
#include <InterViews/event.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>

//
// ToolMenu -- menus for MenuTools
//

Window *ToolMenu::_window;

ToolMenu::~ToolMenu()
{
	Resource::unref(_menu);
}

ToolMenu::ToolMenu(Menu *m)
{
	_menu=m;
	Resource::ref(_menu);
}

//
// MenuTool -- shows a (pulldown) menu at a graphic
//

MenuTool::MenuTool(GraphicMaster *gm,ToolMenu *m): Tool(gm)
{
	_menu=m;
	_popup=nil;
	Resource::ref(_menu);
}

MenuTool::~MenuTool()
{
	Resource::unref(_menu);
}

boolean MenuTool::grasp(const Event &e)
{
	float tol=2.0;
	GraphicMaster *gm=graphic_master();
	BoxObj box(e.pointer_x()-tol, e.pointer_y()-tol, e.pointer_x()+tol, e.pointer_y()+tol);
	Graphic *target=gm->last_intersecting(box);

	if(target!=nil)
	{
		Coord rx=e.pointer_root_x();
		Coord ry=e.pointer_root_y();
		Canvas *c=e.window()->canvas();
		GlyphViewer *gv=gm->glyphviewer();
		Menu *m=_menu->menu();

		_menu->target(target);
		_popup=new PopupWindow(m);
		_popup->style(new Style(WidgetKit::instance()->style()));
		_popup->display(ToolMenu::main_window()->display());
		_popup->place(rx,ry);
		_popup->align(0.5,0.5);
		_popup->map();
		_popup->target(e);
		m->press(e);
		return true;
	}
	return false;
}

boolean MenuTool::manipulating(const Event &e)
{
	if(e.type()==Event::up)
		return false;
	if(1 || e.type()==Event::motion)
	{
		_menu->menu()->drag(e);
	}
	return true;
}

boolean MenuTool::effect(const Event &e)
{
	if(_popup)
	{
		GlyphViewer *gv=graphic_master()->glyphviewer();
		Menu *m=_menu->menu();

		_popup->unmap();
		m->release(e);
		m->unselect();
		delete _popup;
		_popup=nil;
		gv->damage_all();
		gv->repair();
		return true;
	}
	return false;
}

ClassId MenuTool::get_classid() const
{
	return MENU_TOOL;
}

boolean MenuTool::is_a(ClassId id) const
{
	return id==MENU_TOOL||Tool::is_a(id);
}

